import java.io.*;
import java.util.*;

/**
 * class entTable builds hashtables to associate the three types of notation and symbol 
 * references to each other
 *
 * @author Yuzhen Xie, supervised by Dr. Stephen Watt, 08/14/01
 */


public class entTable {
    private Hashtable NameKeyNent;
    private Hashtable NentKeyName;


    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
    /** 
     * Construct an entTable by creating two hashtables to associate named references to numeric entities
     * @return
     */ 
    entTable (String fileName) {
	NameKeyNent = new Hashtable();
	NentKeyName = new Hashtable();

	String [] items;
	items = new String[2];
   	String lineString; //an input line of string

	try {
            BufferedReader NUCData = new BufferedReader(new FileReader(fileName));
	    while ((lineString = NUCData.readLine()) != null) {
		StringTokenizer NUTokens = new StringTokenizer(lineString);
		if ( NUTokens.countTokens() == 2 ) {
		    int itemNum = 0;
		    while ( NUTokens.hasMoreTokens() ) {
			items[itemNum] = NUTokens.nextToken();
			itemNum++;
		    }
		
		    NameKeyNent.put(items[0], getNentByUcode(items[1]));
		    NentKeyName.put(getNentByUcode(items[1]), items[0]);
		}
	    }
	    //System.out.println(NameKeyNent.toString());
	    //System.out.println(NentKeyName.toString());
	    NUCData.close();
	} catch (Exception e) {
	    System.out.println(e);
	}
    }



    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
    /** 
     * Search the unicode by key of a named chars
     * @param String charName, a named chars
     * @return String unicode, null when not found
     */ 
    public String getUcodeByName(String charName) {
	String strNent = getNentByName(charName);
	String strName = null;
	if ( strNent != null )
	    strName = getUcodeByNent(strNent);
	return strName;
    }  


    public String getNameByUcode(String code) {
	String strNent = getNentByUcode(code);
	return getNameByNent(strNent);
    } 


    public String getNameByNent(String ent) {
	return (String)this.NentKeyName.get(ent);
    }  


    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
    /** 
     * Search the numeric ent ref by key of a named chars
     * @param String charName, a named chars
     * @return String unicode, null when not found
     */ 
    public String getNentByName(String charName) {
	return (String)this.NameKeyNent.get(charName);
    }
 

    protected String getUcodeByNent(String ent) {
	String strInt = ent.substring(2, ent.indexOf(';'));
	Integer IntEnt = new Integer(strInt);
	int intEnt = Integer.parseInt(strInt);
	//System.out.println(ent);
	//System.out.println(intEnt);
	String strUcode = "&#x" + Integer.toHexString(intEnt).toUpperCase() + ";";
	return strUcode;
    }

    
    protected String getNentByUcode(String code) {
	String strHex = code.substring(3, code.indexOf(';'));
	//System.out.println(code);
	//System.out.println(strHex);
	String strEnt=null;
	try {
	    Integer IntEnt = new Integer(Integer.parseInt(strHex, 16));
	    int intEnt = IntEnt.intValue();
	    strEnt = "&#" + intEnt + ";";
	}
	catch (NumberFormatException e){
	    System.out.println(e);
	}
	return strEnt;
    }
}
