import java.io.*;
import java.util.*;
import java.lang.*;

/**
 * enttran converts the named characters and numeric entity refs in a XML file to the equivalent 
 * unicode references, or vise versa, based on the given data file.
 * The program should be invoked as "java enttran [html or xml file] [data file] -o [name|nent|ucode] outfile"
 * The data file that contains the notation and symbol references must follow the format of NameUcode.txt as provided.
 * @see CodeTable
 *
 * @author Yuzhen Xie, supervised by Dr. Stephen Watt, 08/15/01
 */

public class enttran {

    public static void main(String[] args) {
	if ( args.length > 0) {
	    String mlInFileName = args[0];
	    String dataFileName = args[1];
	    String converType = args[3];
	    String outFile = args[4];

	    entTable ct = new entTable(dataFileName);

	    if ( converType.equals("name") == true ) 
		NentUcode2NameByFile(mlInFileName, outFile, ct);
	    else if ( converType.equals("nent") == true )
		NameUcode2NentByFile(mlInFileName, outFile, ct);
	    else if ( converType.equals("ucode") == true )
		NameNent2UcodeByFile(mlInFileName, outFile, ct);
	    else
		System.out.println("Invalid command option!");
	}
	else {
	    System.out.println("Invalid command line argumenttranent!");
	}
    }	
    

    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
    /** 
     * Convert named chars and numeric refs to uncodes for a file line by line. It reads the input file line by line, 
     * and writes the converted line to the output file in text format. 
     * @param String inFileName, the name of the XML file to be ocnverted; CodeTable t, an object of
     * of CodeTable contains info of two types of codes.
     * @return
     */ 
    public static void NameNent2UcodeByFile(String inFileName, String mlOutFileName, entTable t) {
	String inLineString;  //a line of the string reading in from input file
	String outLineString1 = null; //a line of converted string
	String outLineString2 = null; //a line of converted string

	try {
            BufferedReader mlInStream = new BufferedReader(new FileReader(inFileName));
	    PrintWriter mlWriter = new PrintWriter(new FileOutputStream(mlOutFileName), true);

	    try {
		while ((inLineString = mlInStream.readLine()) != null) {
		    //System.out.println(inLineString);
		    String firstPart = "";
		    outLineString1 = Name2UcodeByLine(t, inLineString, firstPart);
		    //System.out.println("outLinestring1: " + outLineString1);
		    outLineString2 = Nent2UcodeByLine(t, outLineString1, firstPart);
		    //System.out.println("outLinestring2: " + outLineString2);
		    mlWriter.write(outLineString2);
		    mlWriter.write('\n');
		}
		mlInStream.close();
		mlWriter.close();
		System.out.println("Conversion is done! Output file name is: " + mlOutFileName + '\n');
	    } catch ( Exception e) {
		System.out.println(e);
	    }
	    
	} catch ( Exception e) {
	    System.out.println(e);
	}
    }

    public static void NentUcode2NameByFile(String inFileName, String mlOutFileName, entTable t) {
	String inLineString;  //a line of the string reading in from input file
	String outLineString1 = null; //a line of converted string
	String outLineString2 = null; //a line of converted string

	try {
            BufferedReader mlInStream = new BufferedReader(new FileReader(inFileName));
	    PrintWriter mlWriter = new PrintWriter(new FileOutputStream(mlOutFileName), true);

	    try {
		while ((inLineString = mlInStream.readLine()) != null) {
		    //System.out.println(inLineString);
		    String firstPart = "";
		    outLineString1 = Ucode2NameByLine(t, inLineString, firstPart);
		    outLineString2 = Nent2NameByLine(t, outLineString1, firstPart);
		    mlWriter.write(outLineString2);
		    mlWriter.write('\n');
		}
		mlInStream.close();
		mlWriter.close();
		System.out.println("Conversion is done! Output file name is: " + mlOutFileName + '\n');
	    } catch ( Exception e) {
		System.out.println(e);
	    }
	    
	} catch ( Exception e) {
	    System.out.println(e);
	}
    }


    public static void NameUcode2NentByFile(String inFileName, String mlOutFileName, entTable t) {
	String inLineString;  //a line of the string reading in from input file
	String outLineString1 = null; //a line of converted string
	String outLineString2 = null; //a line of converted string

	try {
            BufferedReader mlInStream = new BufferedReader(new FileReader(inFileName));
            PrintWriter mlWriter = new PrintWriter(new FileOutputStream(mlOutFileName), true);

	    try {
		while ((inLineString = mlInStream.readLine()) != null) {
		    //System.out.println(inLineString);
		    String firstPart = "";
		    outLineString1 = Name2NentByLine(t, inLineString, firstPart);
		    outLineString2 = Ucode2NentByLine(t, outLineString1, firstPart);
		    mlWriter.write(outLineString2);
		    mlWriter.write('\n');
		}
		mlInStream.close();
		mlWriter.close();
		System.out.println("Conversion is done! Output file name is: " + mlOutFileName + '\n');
	    } catch ( Exception e) {
		System.out.println(e);
	    }
	    
	} catch ( Exception e) {
	    System.out.println(e);
	}
    }


  
    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
    /** 
     * Recursively convert named chars to unicodes for a string. It identifies a unicode by symbols '&' 
     * and ';', and replace it with the equivalent unicode found in the CodeTable.  
     * @param CodeTable t, built based on the given code file; String aLine, the string to be converted;
     * String prefix, the front part of the string deliminited by a named char. 
     * @return the converted string
     */ 
    public static String Name2UcodeByLine(entTable t, String aLine, String prefix) {
	String suffix = aLine;
	String last = null;
	int ampersandPos;
	int semicolonPos;
	
	if ( ((ampersandPos = aLine.indexOf("&")) == -1 ) || ((semicolonPos = aLine.indexOf(";")) == -1) ) {
	    last = prefix + suffix;
	    //System.out.println(last);
	    return last;
	} 
	else {
	    char c = aLine.charAt(ampersandPos+1);
	    //System.out.println(c);
	    if ( c == '#' ) {
		prefix = prefix + aLine.substring(0, semicolonPos+1); //include &#
		//System.out.println(prefix);

		suffix = aLine.substring(semicolonPos+1);
		//System.out.println(suffix);
		return Name2UcodeByLine(t, suffix, prefix);
	    }
	    else {
		prefix = prefix + aLine.substring(0, ampersandPos);
		String namedChar = aLine.substring(ampersandPos, semicolonPos+1);//exclude & ;
		//System.out.println(namedChar);
		suffix = aLine.substring(semicolonPos+1); //after ;
		String unicode = t.getUcodeByName(namedChar);
		if ( unicode == null ) {
		    prefix = prefix + namedChar;
		    System.out.println("Cann't convert '" + namedChar + "'! Its Unicode doesn't exist.\n");
		}
		else {
		    prefix = prefix + unicode;
		   
		}
		return Name2UcodeByLine(t, suffix, prefix);
	    }
	}
    }

    public static String Name2NentByLine(entTable t, String aLine, String prefix) {
	String suffix = aLine;
	String last = null;
	int ampersandPos;
	int semicolonPos;
	
	if ( ((ampersandPos = aLine.indexOf("&")) == -1 ) || ((semicolonPos = aLine.indexOf(";")) == -1) ) {
	    last = prefix + suffix;
	    //System.out.println(last);
	    return last;
	} 
	else {
	    char c = aLine.charAt(ampersandPos+1);
	    //System.out.println(c);
	    if ( c == '#' ) {
		prefix = prefix + aLine.substring(0, semicolonPos+1); //include &#
		suffix = aLine.substring(semicolonPos+1);
		return Name2NentByLine(t, suffix, prefix);
	    }
	    else {
		prefix = prefix + aLine.substring(0, ampersandPos);
		String namedChar = aLine.substring(ampersandPos, semicolonPos+1);//exclude & ;
		suffix = aLine.substring(semicolonPos+1); //after ;
		String ent = t.getNentByName(namedChar);
		if ( ent == null ) {
		    prefix = prefix + namedChar;
		    System.out.println("Cann't convert '" + namedChar + "'! Its numeric ent ref doesn't exist.\n");
		}
		else {
		    prefix = prefix + ent;
		   
		}
		return Name2NentByLine(t, suffix, prefix);
	    }
	}
    }



    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
    /** 
     * Recursively convert numeric entity refs to named chars for a string. It identifies them by symbols '&#' 
     * and ';', and replace it with the equivalent named chars found in the CodeTable.  
     * @param CodeTable t, built based on the given code file; String aLine, the string to be converted;
     * String prefix, the front part of the string deliminited by a unicode. 
     * @return the converted string
     */ 
    public static String Nent2NameByLine(entTable t, String aLine, String prefix) {
	String suffix = aLine;
	String last = null;
	int ampersandPos;
	int semicolonPos;
	
	if ( ((ampersandPos = aLine.indexOf("&")) == -1 ) || ((semicolonPos = aLine.indexOf(";")) == -1) ) {
	    last = prefix + suffix;
	    //System.out.println(last);
	    return last;
	} 
	else {
	    //System.out.println(aLine);
	    char c = aLine.charAt(ampersandPos+1);
	    //System.out.println(c);
	    if ( c == '#' ) {
		char s = aLine.charAt(ampersandPos+2);
		if ( s == 'x') {//unicode
		    prefix = prefix + aLine.substring(0, semicolonPos+1);
		    suffix = aLine.substring(semicolonPos+1);
		}
		else {//numeric entity ref
		    prefix = prefix + aLine.substring(0, ampersandPos);
		    String ent = aLine.substring(ampersandPos, semicolonPos+1); //include &#;
		    //System.out.println(ent);
		    suffix = aLine.substring(semicolonPos+1);
	
		    String namedChar = t.getNameByNent(ent);
		    if ( namedChar == null ) {
			prefix = prefix + ent;
			System.out.println("Cann't convert '" + ent + "'! Its Named Char doesn't exist.\n");
		    }
		    else {		
			prefix = prefix + namedChar;		   
		    }
		}
		return Nent2NameByLine(t, suffix, prefix);
	    }
	    else {
		prefix = prefix + aLine.substring(0, semicolonPos+1);
		suffix = aLine.substring(semicolonPos+1);
		return Nent2NameByLine(t, suffix, prefix);
	    }
	}
    }
 

    public static String Nent2UcodeByLine(entTable t, String aLine, String prefix) {
	String suffix = aLine;
	String last = null;
	int ampersandPos;
	int semicolonPos;
	
	if ( ((ampersandPos = aLine.indexOf("&")) == -1 ) || ((semicolonPos = aLine.indexOf(";")) == -1) ) {
	    last = prefix + suffix;
	    //System.out.println(last);
	    return last;
	} 
	else {
	    //System.out.println(aLine);
	    char c = aLine.charAt(ampersandPos+1);
	    //System.out.println(c);
	    if ( c == '#' ) {
		char s = aLine.charAt(ampersandPos+2);
		if ( s == 'x') {//unicode
		    prefix = prefix + aLine.substring(0, semicolonPos+1);
		    suffix = aLine.substring(semicolonPos+1);
		}
		else {//numeric entity ref
		    //System.out.println(aLine);
		    prefix = prefix + aLine.substring(0, ampersandPos);
		    //System.out.println(prefix);
		    String ent = aLine.substring(ampersandPos, semicolonPos+1); //include &#;
		    //System.out.println(ent);
		    suffix = aLine.substring(semicolonPos+1);
		    //System.out.println(suffix);
		    String code = t.getUcodeByNent(ent);
		    if ( code == null ) {
			prefix = prefix + ent;
			System.out.println("Cann't convert '" + ent + "'! Its unicode ref doesn't exist.\n");
		    }
		    else {		
			prefix = prefix + code;		   
		    }
		}
		return Nent2UcodeByLine(t, suffix, prefix);
	    }
	    else {
		//System.out.println("before substring not #");
		prefix = prefix + aLine.substring(0, semicolonPos+1);
		suffix = aLine.substring(semicolonPos+1);
		return Nent2UcodeByLine(t, suffix, prefix);
	    }
	}
    }

    public static String Ucode2NameByLine(entTable t, String aLine, String prefix) {
	String suffix = aLine;
	String last = null;
	int ampersandPos;
	int semicolonPos;
	
	if ( ((ampersandPos = aLine.indexOf("&")) == -1 ) || ((semicolonPos = aLine.indexOf(";")) == -1) ) {
	    last = prefix + suffix;
	    //System.out.println(last);
	    return last;
	} 
	else {
	    //System.out.println(aLine);
	    char c = aLine.charAt(ampersandPos+1);
	    //System.out.println(c);
	    if ( c == '#' ) {
		char s = aLine.charAt(ampersandPos+2);
		if ( s == 'x') {
		    prefix = prefix + aLine.substring(0, ampersandPos);
		    String code = aLine.substring(ampersandPos, semicolonPos+1); //include &#;
		    //System.out.println(ent);
		    suffix = aLine.substring(semicolonPos+1);
	
		    String namedChar = t.getNameByUcode(code);
		    if ( namedChar == null ) {
			prefix = prefix + code;
			System.out.println("Cann't convert '" + code + "'! Its Named Char doesn't exist.\n");
		    }
		    else {		
			prefix = prefix + namedChar;		   
		    }
		}
		else {
		    prefix = prefix + aLine.substring(0, semicolonPos+1);
		    suffix = aLine.substring(semicolonPos+1);
		}
		return Ucode2NameByLine(t, suffix, prefix);
	    }
	    else {
		prefix = prefix + aLine.substring(0, semicolonPos+1);
		suffix = aLine.substring(semicolonPos+1);
		return Ucode2NameByLine(t, suffix, prefix);
	    }
	}

    }


    public static String Ucode2NentByLine(entTable t, String aLine, String prefix) {
	String suffix = aLine;
	String last = null;
	int ampersandPos;
	int semicolonPos;
	
	if ( ((ampersandPos = aLine.indexOf("&")) == -1 ) || ((semicolonPos = aLine.indexOf(";")) == -1) ) {
	    last = prefix + suffix;
	    //System.out.println(last);
	    return last;
	} 
	else {
	    //System.out.println(aLine);
	    char c = aLine.charAt(ampersandPos+1);
	    //System.out.println(c);
	    if ( c == '#' ) {
		char s = aLine.charAt(ampersandPos+2);
		if ( s == 'x') {
		    prefix = prefix + aLine.substring(0, ampersandPos);
		    String code = aLine.substring(ampersandPos, semicolonPos+1); //include &#;
		    //System.out.println(ent);
		    suffix = aLine.substring(semicolonPos+1);
	
		    String ent = t.getNentByUcode(code);
		    if ( ent == null ) {
			prefix = prefix + code;
			System.out.println("Cann't convert '" + code + "'! Its numeric ent ref doesn't exist.\n");
		    }
		    else {		
			prefix = prefix + ent;		   
		    }
		}
		else {
		    prefix = prefix + aLine.substring(0, semicolonPos+1);
		    suffix = aLine.substring(semicolonPos+1);
		}
		return Ucode2NentByLine(t, suffix, prefix);
	    }
	    else {
		prefix = prefix + aLine.substring(0, semicolonPos+1);
		suffix = aLine.substring(semicolonPos+1);
		return Ucode2NentByLine(t, suffix, prefix);
	    }
	}
    }
}



    
